library struct_transfer;

import "dart:async";
import "dart:convert";


class TransferBuilder{
  
  final List<StructBuilder> _sbs = [];
  Function _onFromTransferData;
  
  TransferBuilder(Stream<String> to_transfer_stream, void this._onFromTransferData(String str)){
    to_transfer_stream.listen(_onData);
  }
  
  StructBuilder getStructBuilder(){
    var sb = new StructBuilder._(this);
    _sbs.add(sb);
    return sb;
  }
  
  void _send(StructBuilder sb , List<List<int>> l){
    //あとでちゃんとバイト配列にする。
    _onFromTransferData(JSON.encode([_sbs.indexOf(sb),l]));
  }
  
  void _onData(String str){
    List l = JSON.decode(str);
    _sbs[l[0]]._onData(l[1]);
  }
  
}

abstract class Transfer{
  
  final TransferBuilder _tb;
  
  Transfer(this._tb);
  
}

class StructBuilder{
  
  final List<Param> _params = []; 
  final TransferBuilder _tb;
  final StreamController _sc = new StreamController();
  
  StructBuilder._(this._tb);
  
  StringParam stringParam() => _add(new StringParam._());
  IntParam intParam() => _add(new IntParam._());
  //DoubleParam doubleParam() => _add(new DoubleParam._());
  DateTimeParam dateTimeParam() => _add(new DateTimeParam._());
//  ListParam listParam() => _add(new ListParam._());
  
  get _stream => _sc.stream;
  
  Param _add(Param param){
    _params.add(param);
    return param;
  }
  
  void _send(){
    List<List<int>> l = [];
    _params.forEach((e) => l.add(e._toInts));
    _tb._send(this,l);
    
  }
  
  void _onData(List<List<int>> l){
    for(int x = 0 ; x < _params.length ; x ++){
      _params[x]._fromInts = l[x];
    }
    
    _sc.add(null);
    
  }
  
}

abstract class Struct{
  
  final StructBuilder _sb;
  Struct(this._sb);
  
  void send(){
    _sb._send();
  }
  
  Stream get onData => _sb._stream;
  
}

abstract class Param<T>{
  T _value;
  T get value => _value;
  set value(T t) => _value = t;
  
  List<int> get _toInts;
  set _fromInts(List<int> i);
  
}

class IntParam extends Param<int>{
  IntParam._();
  List<int>get _toInts => [_value];
  set _fromInts(List<int> i) => _value = i[0];
  
}

class DateTimeParam extends Param<DateTime>{
  DateTimeParam._();
  List<int> get _toInts => [_value.millisecondsSinceEpoch];
  set _fromInts(List<int> i) => _value = new DateTime.fromMillisecondsSinceEpoch(i[0]); 
  
}

class StringParam extends Param<String>{
  StringParam._();
  List<int> get _toInts => _value.codeUnits;
  set _fromInts(List<int> i) => _value = new String.fromCharCodes(i);
  
}

//class DoubleParam extends Param<double>{
//  
//}

//class ListParam extends Param<List<Param>>{
//  ListParam._();
//}